import os
import threading
from datetime import datetime

import numpy as np
from pydicom import dcmread, uid


def merge_doses(root, frame, progressbar, button, output, frame2, log):

    output.add_text("Begun job...")

    try:

        initial_file = dcmread(frame.simulations[0])

        try:
            initial_file.SeriesDescription = frame.newseriesdescription
        except Exception:
            try:
                initial_file.SeriesDescription = frame.seriesdescriptionentry.get()
                frame.newseriesdescription = initial_file.SeriesDescription
                if frame.newseriesdescription == "":
                    raise Exception
                output.add_text(
                    f"Set output series description to {frame.newseriesdescription}"
                )
            except Exception:
                output.add_text("Invalid series description!")
                return

        hist_cal = frame.histories
        try:
            hist_sum = frame.simulated_histories
        except Exception:
            try:
                hist_sum = int(frame.simhistoriesentry.get())
                frame.simulated_histories = hist_sum
                output.add_text(f"Set simulated histories to: {hist_sum}")
            except Exception:
                output.add_text("Invalid simulated histories entry!")
                return

        dose_cal = frame.avg_dose
        fx = frame.fractions

        scale = (
            initial_file.DoseGridScaling
            * 1
            / dose_cal
            * frame.mus[0]
            / 100
            * hist_cal
            / hist_sum
            * fx
        )

        array = initial_file.pixel_array * scale

        button.grid_forget()
        progressbar.grid(row=2, column=0, columnspan=4, padx=(5, 5), pady=(5, 5))
        log.grid(row=2, column=4, padx=(5, 5), pady=(5, 5))

        progressbar["value"] += 100 / len(frame.simulations)

        threading.Thread(target=lambda: frame2.calculate_dvhs()).start()

        def run_merge(frame, array):
            for i in range(1, len(frame.simulations)):

                with dcmread(frame.simulations[i]) as dicom:
                    scale = (
                        dicom.DoseGridScaling
                        * 1
                        / dose_cal
                        * frame.mus[i]
                        / 100
                        * hist_cal
                        / hist_sum
                        * fx
                    )
                    array += dicom.pixel_array * scale

                progressbar["value"] += 100 / len(frame.simulations)
                root.update_idletasks()

        threading.Thread(target=run_merge(frame, array)).start()
        root.update()

        initial_file.BitsAllocated = 32
        initial_file.BitsStored = 32
        initial_file.HighBit = 31

        initial_file.DoseGridScaling = np.max(array) / (2 ** int(initial_file.HighBit))

        pixel_array_summed = (array / initial_file.DoseGridScaling).astype(np.uint32)

        initial_file.PixelData = pixel_array_summed.tobytes()
        initial_file.SeriesDescription = frame.newseriesdescription

        dt = datetime.now()
        initial_file.InstanceCreationDate = dt.strftime("%Y%m%d")
        initial_file.ContentDate = dt.strftime("%Y%m%d")
        timeStr = dt.strftime("%H%M%S.%f")  # long format with micro seconds
        initial_file.InstanceCreationTime = timeStr
        initial_file.ContentTime = timeStr
        initial_file.SOPInstanceUID = uid.generate_uid(
            prefix=".".join(initial_file.SOPInstanceUID.split(".")[:-1]) + "."
        )
        initial_file.DoseUnits = "Gy"
        initial_file.DoseType = "PHYSICAL"
        initial_file.DoseSummationType = "PLAN"
        filename = frame.newseriesdescription + ".dcm"
        initial_file.save_as(f"{os.path.join(frame.folder_selected, filename)}")
        output.add_text(
            f"Saving {frame.newseriesdescription}.dcm to {frame.folder_selected}"
        )

        progressbar["value"] = 0

        output.add_text("(SUCCESS) All simulations succesfully scaled and merged!")

    except Exception as e:
        print(e)
        if e.__str__() == "'Configurator' object has no attribute 'simulations'":
            output.add_text("(ERROR) No or invalid simulation directory selected!")

        if e.__str__() == "'Configurator' object has no attribute 'mus'":
            output.add_text("(ERROR) No or invalid DICOM directory selected!")

        if e.__str__() == "'Configurator' object has no attribute 'histories'":
            output.add_text("(ERROR) No or invalid reference simulation file selected!")
        return

    return




import os
import threading
import time
import tkinter as tk
import tkinter.ttk as ttk

import dicompylercore.dvhcalc as dv
import matplotlib as mpl

mpl.use("Agg")
import matplotlib.pyplot as plt
from pydicom import dcmread


class StructureSelector(tk.Frame):
    def __init__(self, parenttk, parent):

        tk.Frame.__init__(self, parenttk)
        self.parenttk = parenttk
        self.parent = parent

        self.configure(
            highlightbackground="black", highlightthickness=2, highlightcolor="black"
        )

    def create_buttons(self):

        self.structures = []
        with dcmread(self.parent.frame.rtstruct) as dcmfile:
            for i in range(len(dcmfile.StructureSetROISequence)):
                self.structures += [
                    (
                        dcmfile.StructureSetROISequence[i].ROINumber,
                        dcmfile.StructureSetROISequence[i].ROIName,
                    )
                ]
        self.structures.sort(key=lambda y: y[0])

        self.variables = [tk.BooleanVar() for i in range(len(self.structures))]
        [variable.set(False) for variable in self.variables]
        self.buttons = [
            ttk.Checkbutton(
                self, variable=self.variables[i], text=f"{self.structures[i][1]}"
            )
            for i in range(len(self.structures))
        ]

    def show_buttons(self):

        try:

            if self.parent.frame.dvh.get() == True:

                [
                    button.grid(sticky="W", padx=(2, 2), pady=(1, 1))
                    for button in self.buttons
                ]
            else:
                [button.grid_forget() for button in self.buttons]

        except AttributeError:

            self.parent.output.add_text(
                "(ERROR) No or invalid RTSTRUCT file in reference DICOM folder!"
            )
            self.parent.frame.dvh.set(False)

    def calculate_dvhs(self):

        if self.parent.frame.dvh.get() == False:
            self.parent.pb.grid_forget()
            self.parent.log.grid_forget()
            self.parent.log.configure(text="Merging Sims ...")
            self.parent.pb["value"] = 0
            self.parent.run.grid(
                row=2, column=1, columnspan=2, padx=(5, 5), pady=(5, 5)
            )
            self.parent.output.add_text(f"(SUCCESS) Completed!")
            return

        def calculate_TOPAS_DVHs():
            dvh = []
            filename = self.parent.frame.newseriesdescription + ".dcm"
            filename = os.path.join(self.parent.frame.folder_selected, filename)

            while True:
                if os.path.exists(filename):
                    for i in range(len(self.structures)):
                        if self.variables[i].get() == True:
                            self.parent.output.add_text(
                                f"Calculating TOPAS DVH for structure {self.structures[i][1]} ..."
                            )
                            dvh += [
                                dv.get_dvh(
                                    self.parent.frame.rtstruct,
                                    filename,
                                    self.structures[i][0],
                                )
                            ]
                            self.parent.pb["value"] += 50 / len(
                                [
                                    variable
                                    for variable in self.variables
                                    if variable.get() == True
                                ]
                            )

                if len(dvh) == len([x for x in self.variables if x.get() == True]):
                    break
            self.topas_dvh = dvh

        def calculate_Reference_DVHs():
            dvh = []

            for i in range(len(self.structures)):
                if self.variables[i].get() == True:
                    self.parent.output.add_text(
                        f"Calculating Reference DVH for structure {self.structures[i][1]} ..."
                    )
                    dvh += [
                        dv.get_dvh(
                            self.parent.frame.rtstruct,
                            f"{self.parent.frame.refdose}",
                            self.structures[i][0],
                        )
                    ]
                    self.parent.pb["value"] += 50 / len(
                        [
                            variable
                            for variable in self.variables
                            if variable.get() == True
                        ]
                    )
            self.ref_dvh = dvh

        self.parent.log.configure(text="Creating DVHs ...")
        threading.Thread(calculate_TOPAS_DVHs()).start()
        threading.Thread(calculate_Reference_DVHs()).start()

        while True:
            try:
                if self.ref_dvh:
                    pass
                if self.topas_dvh:
                    pass

                for i in range(len(self.ref_dvh)):
                    plt.plot(
                        self.topas_dvh[i].bincenters,
                        self.topas_dvh[i].relative_volume.counts,
                        label=self.topas_dvh[i].name + " - TOPAS",
                    )

                    plt.plot(
                        self.ref_dvh[i].bincenters,
                        self.ref_dvh[i].relative_volume.counts,
                        label=self.ref_dvh[i].name + " - Ref",
                    )

                break

            except NameError:
                pass

        plt.xlabel("Dose [%s]" % self.ref_dvh[0].dose_units)
        plt.ylabel("Volume [%s]" % self.ref_dvh[0].relative_volume.volume_units)
        plt.legend(loc="best")
        self.parent.output.add_text(
            f"Saving DVH.png to {self.parent.frame.folder_selected}"
        )
        plt.savefig(
            os.path.join((self.parent.frame.folder_selected), "DVH.png"), dpi=600
        )

        self.parent.pb.grid_forget()
        self.parent.log.grid_forget()
        self.parent.log.configure(text="Merging Sims ...")
        self.parent.pb["value"] = 0
        self.parent.run.grid(row=2, column=1, columnspan=2, padx=(5, 5), pady=(5, 5))
        self.parent.output.add_text(f"(SUCCESS) Completed!")
        return




import tkinter as tk
from datetime import datetime
from tkinter import font


class Output(tk.Frame):
    def __init__(self, parent):

        tk.Frame.__init__(self, parent)
        self.parent = parent

        self.configure(
            highlightbackground="black", highlightthickness=2, highlightcolor="black",
        )

        self.scrollbar = tk.Scrollbar(self)
        self.scrollbar.pack(side=tk.RIGHT, fill="y")
        self.textbox = tk.Text(self, width=70, wrap=tk.WORD)
        self.textbox.pack(side=tk.TOP, fill="both", expand=True)
        self.textbox.config(yscrollcommand=self.scrollbar.set)
        self.textbox.configure(state="disabled")
        self.scrollbar.config(command=self.textbox.yview)

        text_font = font.nametofont(self.textbox.cget("font"))
        bullet_width = text_font.measure("%H:%M:%S - ")
        em = text_font.measure("m")
        self.textbox.tag_configure("bulleted", lmargin1=em, lmargin2=em + bullet_width)

    def add_text(self, text=None):

        now = datetime.now()

        current_time = now.strftime("%H:%M:%S")
        self.textbox.configure(state="normal")
        self.textbox.insert(tk.END, f"{current_time} - {text}\n", "bulleted")
        self.textbox.configure(state="disabled")
        self.textbox.see("end")





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat May 28 10:54:31 2022

@author: Sebastian Schäfer
@institution: Martin-Luther-Universität Halle-Wittenberg
@email: sebastian.schaefer@student.uni-halle.de
"""

import os
import threading
import tkinter as tk
import tkinter.ttk as ttk

from .src.config import Configurator
from .src.merge_doses import merge_doses
from .src.output import Output
from .src.structure_selector import StructureSelector


class TopasDoseCalc:
    def __init__(self):

        self.root = tk.Tk()

        screen_width = self.root.winfo_screenwidth()
        screen_height = self.root.winfo_screenheight()
        width = 812
        height = 550
        self.root.minsize(width, height)

        x = screen_width // 2 - width // 2
        y = screen_height // 2 - height // 2
        self.root.geometry(f"{width}x{height}+{x-25}+{y}")
        self.root.state("normal")
        self.root.title("TopasDoseCalc")
        self.root.tk.call(
            "wm",
            "iconphoto",
            self.root._w,
            tk.PhotoImage(
                file=os.path.realpath(
                    os.path.join(
                        os.path.dirname(os.path.realpath(__file__)), "src", "icon.png",
                    )
                )
            ),
        ),

        ttk.Style(self.root)
        self.root.tk.call(
            "source",
            str(
                os.path.join(
                    os.path.dirname(os.path.dirname(os.path.realpath(__file__))),
                    "topasdosecalc",
                    "src",
                    "Azure-ttk-theme",
                    "azure.tcl",
                )
            ),
        )
        self.frame = Configurator(self.root, self)
        self.frame.grid(row=0, column=0, columnspan=5, padx=(5, 5), pady=(5, 0))
        self.root.columnconfigure(4, weight=10)
        self.frame2 = StructureSelector(self.root, self)
        self.frame2.grid(row=1, column=4, padx=(5, 5), pady=(5, 5))
        self.output = Output(self.root)
        self.output.add_text("Initialized")
        self.output.grid(row=1, column=0, columnspan=4, padx=(5, 5), pady=(5, 5))
        self.pb = ttk.Progressbar(
            self.root, orient=tk.HORIZONTAL, length=400, mode="determinate"
        )
        self.log = tk.Label(self.root, text="Loading DICOMs ...")
        self.run = ttk.Button(
            self.root,
            text="RUN!",
            command=lambda: threading.Thread(
                target=lambda: merge_doses(
                    self.root,
                    self.frame,
                    self.pb,
                    self.run,
                    self.output,
                    self.frame2,
                    self.log,
                )
            ).start(),
        )
        self.run.grid(row=2, column=1, columnspan=2, padx=(5, 5), pady=(5, 5))

    def mainloop(self):
        self.root.mainloop()


def topasdosecalc():
    TDC = TopasDoseCalc()
    TDC.mainloop()


if __name__ == "__main__":
    topasdosecalc()



import os
import threading
import tkinter as tk
import tkinter.ttk as ttk
from tkinter import filedialog as fd
from tkinter import simpledialog as sd

import numpy as np
import topas2numpy
from dicompylercore import dicomparser
from natsort import natsorted, ns
from PIL import Image, ImageTk
from pydicom import dcmread


class Configurator(tk.Frame):
    def __init__(self, parenttk, parent):

        tk.Frame.__init__(self, parenttk)
        self.parent = parent
        self.parenttk = parenttk

        self.rowconfigure(0, weight=1)
        self.rowconfigure(1, weight=1)
        self.rowconfigure(2, weight=1)
        self.rowconfigure(3, weight=1)
        self.rowconfigure(4, weight=1)
        self.columnconfigure(0, weight=1)
        self.columnconfigure(1, weight=1)
        self.columnconfigure(2, weight=1)
        self.columnconfigure(3, weight=1)

        self.configure(
            highlightbackground="black", highlightthickness=2, highlightcolor="black"
        )

        buttonImage = Image.open(
            os.path.realpath(
                os.path.join(
                    os.path.dirname(os.path.dirname(os.path.realpath(__file__))),
                    "src",
                    "folder.ico",
                )
            )
        )
        self.foldertext = tk.Canvas(self, width=200, height=20)
        self.foldertext.create_text(
            100, 10, anchor=tk.CENTER, text="1. Select TOPAS simulation directory"
        )
        self.foldertext.grid(column=0, row=0)  # , rowspan=2)
        self.buttonPhoto = ImageTk.PhotoImage(buttonImage)
        self.dcmfolderbutton = ttk.Button(
            self,
            text="TOPAS simulation results",
            image=self.buttonPhoto,
            compound=tk.TOP,
            command=lambda: threading.Thread(target=self.pick_sim_folder).start(),
        )
        self.dcmfolderbutton.grid(column=0, row=1, rowspan=4)

        self.reftext = tk.Canvas(self, width=200, height=20)
        self.reftext.create_text(
            100, 10, anchor=tk.CENTER, text="2. Select TOPAS reference simulation"
        )
        self.reftext.grid(column=1, row=0)  # , rowspan=2)
        topasImage = Image.open(
            os.path.realpath(
                os.path.join(
                    os.path.dirname(os.path.dirname(os.path.realpath(__file__))),
                    "src",
                    "topas.ico",
                )
            )
        )
        self.topasPhoto = ImageTk.PhotoImage(topasImage)
        self.refbutton = ttk.Button(
            self,
            text="TOPAS reference simulation",
            image=self.topasPhoto,
            compound=tk.TOP,
            command=lambda: threading.Thread(target=self.pick_ref_simulation).start(),
        )
        self.refbutton.grid(column=1, row=1, rowspan=4)

        self.refdcmtext = tk.Canvas(self, width=200, height=20)
        self.refdcmtext.create_text(
            100, 10, anchor=tk.CENTER, text="3. Select CT, RTPLAN and RTSTRUCT"
        )
        self.refdcmtext.grid(column=2, row=0)  # , rowspan=2)
        dcmImage = Image.open(
            os.path.realpath(
                os.path.join(
                    os.path.dirname(os.path.dirname(os.path.realpath(__file__))),
                    "src",
                    "dcm.ico",
                )
            )
        )
        self.dcmPhoto = ImageTk.PhotoImage(dcmImage)
        self.refbutton = ttk.Button(
            self,
            text="CT, RTPLAN, RTSTRUCT",
            image=self.dcmPhoto,
            compound=tk.TOP,
            command=lambda: threading.Thread(target=self.pick_ref_dicom_folder).start(),
        )
        self.refbutton.grid(column=2, row=1, rowspan=4)

        self.simhistories = tk.Label(self, text="4. Histories per simulation:")
        self.simhistoriesentry = tk.Entry(self, width=30)

        self.simhistories.grid(column=3, row=0)
        self.simhistoriesentry.grid(column=3, row=1, padx=(0, 5))

        self.seriesdescription = tk.Label(self, text="5. New DICOM series description:")
        self.seriesdescriptionentry = tk.Entry(self, width=30)

        self.seriesdescription.grid(column=3, row=2)
        self.seriesdescriptionentry.grid(column=3, row=3, padx=(0, 5))
        self.parenttk.bind("<Return>", self.pick_simulated_histories)

        self.dvh = tk.BooleanVar()
        self.dvh.set(False)
        self.dvhselect = ttk.Checkbutton(
            self,
            text="Calculate DVHs ?",
            command=lambda: threading.Thread(
                target=self.parent.frame2.show_buttons()
            ).start(),
            variable=self.dvh,
        )
        self.dvhselect.grid(column=3, row=4)

    def select_structures(self):
        if self.dvh.get() == True:
            print(True)
        else:
            print(False)

    def pick_sim_folder(self):
        self.folder_selected = fd.askdirectory()
        if self.folder_selected != "":
            self.parent.output.add_text(
                f"Selected simulation directory: {self.folder_selected}"
            )
            self.parent.output.add_text(
                f"Searching for TOPAS RTDOSE simulation results..."
            )
            self.simulations = []

            self.parent.run.grid_forget()
            self.parent.pb.grid(row=2, column=0, columnspan=4, padx=(5, 5), pady=(5, 5))

            self.parent.log.grid(row=2, column=4, padx=(5, 5), pady=(5, 5))

            for r, d, f in os.walk(self.folder_selected):
                for file in f:
                    if file.endswith("dcm"):
                        # with dcmread(os.path.join(r, file)) as dcmfile:
                        dcmfile = dicomparser.DicomParser(os.path.join(r, file))
                        if dcmfile.ds.Modality == "RTDOSE":
                            self.simulations += [os.path.join(r, file)]

                            self.parenttk.update_idletasks()
                            self.parent.pb["value"] += 100 / len(next(os.walk(r))[2])

            self.parent.log.grid_forget()
            self.parent.log.configure(text="Merging sims ...")
            self.parent.pb.grid_forget()
            self.parent.pb.config(mode="determinate")
            self.parent.run.grid(
                row=2, column=1, columnspan=2, padx=(5, 5), pady=(5, 5)
            )
            self.parent.pb["value"] = 0

            self.parent.output.add_text(f"Found {len(self.simulations)} RTDOSE files!")
            self.simulations = natsorted(self.simulations, alg=ns.IC)
        else:
            self.parent.output.add_text("Simulation folder selection cancelled!")

    def pick_ref_simulation(self):
        self.reference_simulation = fd.askopenfilename(
            filetypes=[("TOPAS simulation outputs", "*.csv; *.bin")]
        )
        if self.reference_simulation != "":
            self.parent.output.add_text(
                f"Selected reference simulation: {self.reference_simulation}"
            )
            try:
                self.data = topas2numpy.BinnedResult(self.reference_simulation)
            except Exception:
                self.parent.output.add_text(f"Unsupported reference simulation type!")
                self.reference_simulation = ""
                return
            bins = [dim.n_bins for dim in self.data.dimensions]
            if bins.count(1) != 2:
                self.parent.output.add_text(f"Unsupported reference simulation type!")
                self.reference_simulation = ""
                return

            axdict = {0: "X", 1: "Y", 2: "Z"}
            databin_index = bins.index(max(bins))
            self.direction = axdict[databin_index]
            self.axis = np.array(self.data.dimensions[databin_index].get_bin_centers())
            unit = self.data.dimensions[databin_index].unit
            self.axis = [self.convert_SI(x, unit) for x in self.axis]
            if "Mean" in self.data.statistics:
                scored_quantity = "Mean"
            if "Sum" in self.data.statistics:
                scored_quantity = "Sum"
            self.dose = self.data.data[scored_quantity]
            self.dose = np.flip(self.dose.flatten())
            try:
                self.std_dev = np.flip(self.data.data["Standard_Deviation"].flatten())
                self.histories = int(
                    self.data.data["Histories_with_Scorer_Active"].flatten()[0]
                )
                if scored_quantity == "Sum":
                    self.std_dev = self.std_dev * np.sqrt(self.histories)
                else:
                    self.std_dev = self.std_dev / np.sqrt(self.histories)
            except KeyError as e:
                if e.args[0] == "Histories_with_Scorer_Active":
                    self.histories = sd.askinteger(
                        title="  ", prompt="Histories used for reference simulation:"
                    )
                    self.std_dev = self.data.data["Standard_Deviation"].flatten()
                    if scored_quantity == "Sum":
                        self.std_dev = self.std_dev * np.sqrt(self.histories)
                    else:
                        self.std_dev = self.std_dev / np.sqrt(self.histories)
                else:
                    self.std_dev = np.array([])

            self.parent.output.add_text(
                f"Reference simulation in {self.direction}-direction with binning {tuple(bins)}"
            )
            self.parent.output.add_text(
                f"Reference simulation used {self.histories} total histories"
            )
            self.avg_dose = np.mean(
                self.dose[len(self.dose) // 2 - 3 : len(self.dose) // 2 + 3]
            )
            self.parent.output.add_text(f"Setting reference dose to {self.avg_dose} Gy")

        else:
            self.parent.output.add_text("Reference simulation selection cancelled!")

    def pick_ref_dicom_folder(self):
        self.dcm_folder_selected = fd.askdirectory()
        if self.dcm_folder_selected != "":
            self.parent.output.add_text(
                f"Selected reference DICOM directory: {self.dcm_folder_selected}"
            )
            CTs = 0
            for r, d, f in os.walk(self.dcm_folder_selected):
                for file in f:
                    with dcmread(os.path.join(r, file)) as dcmfile:
                        if dcmfile.Modality == "CT":
                            CTs += 1
                        elif dcmfile.Modality == "RTDOSE":
                            self.refdose = os.path.join(r, file)
                            self.parent.output.add_text(
                                f"Selected reference RTDOSE file: {file}"
                            )
                        elif dcmfile.Modality == "RTSTRUCT":
                            self.rtstruct = os.path.join(r, file)
                            threading.Thread(
                                target=lambda: self.parent.frame2.create_buttons()
                            ).start()
                            self.parent.output.add_text(
                                f"Selected reference RTSTRUCT file: {file}"
                            )
                        elif dcmfile.Modality == "RTPLAN":
                            self.rtplan = os.path.join(r, file)
                            self.fractions = 0
                            for frac in dcmfile.FractionGroupSequence:
                                self.fractions += int(frac.NumberOfFractionsPlanned)
                            self.total_mu = []
                            for mu in dcmfile.FractionGroupSequence:
                                for beam in mu.ReferencedBeamSequence:
                                    self.total_mu += [float(beam.BeamMeterset)]
                            self.mus = []
                            for j in range(len(dcmfile.BeamSequence)):
                                for i in range(
                                    len(dcmfile.BeamSequence[j].ControlPointSequence)
                                ):
                                    self.mus += [
                                        float(
                                            dcmfile.BeamSequence[j]
                                            .ControlPointSequence[i]
                                            .CumulativeMetersetWeight
                                        )
                                        * self.total_mu[j]
                                    ]

                            self.mus = np.diff(self.mus).tolist()
                            self.mus.append(0)
                            self.total_mu = sum(self.total_mu)
                            for index, value in enumerate(self.mus):
                                if value < 0:
                                    self.mus[index] = 0

                            self.parent.output.add_text(
                                f"Selected reference RTPLAN file: {file}"
                            )
                            self.parent.output.add_text(
                                f"Number of planned fractions: {self.fractions}"
                            )
                            self.parent.output.add_text(
                                f"Total MUs/fraction: {self.total_mu}"
                            )

        else:
            self.parent.output.add_text("Reference DICOM folder selection cancelled!")

    def pick_simulated_histories(self, event=None):
        if event.widget.winfo_name() != "!entry2":

            self.simulated_histories = self.simhistoriesentry.get()
            try:
                self.simulated_histories = int(self.simulated_histories)
                self.parent.output.add_text(
                    f"Set simulated histories to: {self.simulated_histories}"
                )
            except Exception:
                self.parent.output.add_text("Invalid simulated histories entry!")
        else:
            self.newseriesdescription = self.seriesdescriptionentry.get()
            if self.newseriesdescription == "":
                self.parent.output.add_text("Invalid series description!")
            else:
                self.parent.output.add_text(
                    f"Set output series description to {self.newseriesdescription}"
                )

    def convert_SI(self, val, unit_in):
        SI = {"mm": 0.001, "cm": 0.01, "m": 1.0, "km": 1000.0}
        return val * SI[unit_in] / SI["mm"]




import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="topasdosecalc",
    version="1.2",
    author="Sebastian Schäfer",
    author_email="sebastian.schaefer@student.uni-halle.de",
    description="Merge and scale TOPAS DICOMS - calculate and compare DVHs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/sebasj13/TopasDoseCalc",
    project_urls={"Bug Tracker": "https://github.com/sebasj13/TopasDoseCalc/issues",},
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "numpy",
        "matplotlib",
        "topas2numpy",
        "pydicom",
        "dicompyler-core",
        "natsort",
        "Pillow",
        "topas2numpy",
    ],
    packages=["topasdosecalc", "topasdosecalc.src"],
    scripts=["topasdosecalc/topasdosecalc.py"],
    entry_points={
        "console_scripts": ["topasdosecalc=topasdosecalc.topasdosecalc:topasdosecalc"],
    },
    keywords=["topas", "monte-carlo", "python", "simulation", "dvh", "dicom"],
    python_requires=">=3.8",
)
